// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMATLIST__IPP
#define BOOST__IO__FORMATLIST__IPP
   template< typename ForwardIterator, typename FormatType, typename Outputter >
   ForwardIterator boost::io::formatlist_t< ForwardIterator, FormatType, Outputter >::begin() const throw()
   {
      return( first );
   }
   template< typename ForwardIterator, typename FormatType, typename Outputter >
   ForwardIterator boost::io::formatlist_t< ForwardIterator, FormatType, Outputter >::end() const throw()
   {
      return( last );
   }

   template< typename ForwardIterator, typename FormatType, typename Outputter >
   boost::io::formatlist_t< ForwardIterator, FormatType, Outputter >::formatlist_t
   (
      const formatlist_t & fl
   ) throw():
      detail::list_output< FormatType, formatlist_t, Outputter >( fl ),
      first( fl.first ),
      last(  fl.last )
   {
   }
   template< typename ForwardIterator, typename FormatType, typename Outputter >
   boost::io::formatlist_t< ForwardIterator, FormatType, Outputter >::formatlist_t
   (
      ForwardIterator f,
      ForwardIterator l
   ) throw():
      first( f ),
      last(  l )
   {
   }
   template< typename ForwardIterator, typename FormatType, typename Outputter >
   boost::io::formatlist_t< ForwardIterator, FormatType, Outputter >::formatlist_t
   (
      ForwardIterator f,
      ForwardIterator l,
      Outputter       out
   ) throw():
      detail::list_output< FormatType, formatlist_t, Outputter >( out ),
      first( f ),
      last(  l )
   {
   }

   template< typename CharT, class TraitsT, typename ForwardIterator, typename FormatType, typename Outputter >
   std::basic_ostream< CharT, TraitsT > & boost::io::operator<<
   (
      std::basic_ostream< CharT, TraitsT > & os,
      const boost::io::formatlist_t< ForwardIterator, FormatType, Outputter > & fl
   )
   {
      return( fl( os, fl.begin(), fl.end()));
   }

   // basic format manipulator:

   template< typename ForwardIterator >
   boost::io::formatlist_t< ForwardIterator > boost::io::formatlist
   (
      ForwardIterator first,
      ForwardIterator last
   )
   {
      return( formatlist_t< ForwardIterator >( first, last ));
   }

   template< class Container >
   boost::io::formatlist_t< typename Container::const_iterator > boost::io::formatlist
   (
      const Container & c
   )
   {
      return( formatlist_t< typename Container::const_iterator >( c.begin(), c.end()));
   }

   // advanced format manipulator:

   template< typename ForwardIterator, typename FormatType, class RefType >
   boost::io::formatlist_t< ForwardIterator, FormatType > boost::io::formatlist
   (
      ForwardIterator first,
      ForwardIterator last,
      const formatter_t< FormatType, RefType > & fmt
   )
   {
      return( formatlist_t< ForwardIterator, FormatType >( first, last, fmt ));
   }

   template< class Container, typename FormatType, class RefType >
   boost::io::formatlist_t< typename Container::const_iterator, FormatType > boost::io::formatlist
   (
      const Container & c,
      const formatter_t< FormatType, RefType > & fmt
   )
   {
      return( formatlist_t< typename Container::const_iterator, FormatType >( c.begin(), c.end(), fmt ));
   }

   // extended format manipulator:

   template< typename FormatType, typename ForwardIterator >
   boost::io::formatlist_t< ForwardIterator, FormatType > boost::io::formatlistex
   (
      ForwardIterator first,
      ForwardIterator last
   )
   {
      return( formatlist_t< ForwardIterator, FormatType >( first, last ));
   }

   template< typename FormatType, class Container >
   boost::io::formatlist_t< typename Container::const_iterator, FormatType > boost::io::formatlistex
   (
      const Container & c
   )
   {
      return( formatlist_t< typename Container::const_iterator, FormatType >( c.begin(), c.end()));
   }

   // format manipulator -- output functor versions:    

   template< typename ForwardIterator, class Outputter >
   boost::io::formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
   boost::io::formatlistfn
   (
      ForwardIterator first,
      ForwardIterator last,
      Outputter       out
   )
   {
      return( formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
      (
         first,
         last,
         out
      ));
   }

   template< class Container, class Outputter >
   boost::io::formatlist_t< typename Container::const_iterator, typename Outputter::listfmt_type, Outputter >
   boost::io::formatlistfn
   (
      const Container & c,
      Outputter out
   )
   {
      return( formatlist_t< typename Container::const_iterator, typename Outputter::listfmt_type, Outputter >
      (
         c.begin(),
         c.end(),
         out
      ));
   }

   template< typename ForwardIterator, typename FormatType, class Outputter, class RefType >
   boost::io::formatlist_t< ForwardIterator, FormatType, Outputter >
   boost::io::formatlistfn2
   (
      ForwardIterator first,
      ForwardIterator last,
      Outputter       out,
      const formatter_t< FormatType, RefType > & fmt
   )
   {
      return( formatlist_t< ForwardIterator, FormatType, Outputter >
      (
         first,
         last,
         fmt,
         out
      ));
   }

   template< class Container, typename FormatType, class Outputter, class RefType >
   boost::io::formatlist_t< typename Container::const_iterator, FormatType, Outputter >
   boost::io::formatlistfn2
   (
      const Container & c,
      Outputter out,
      const formatter_t< FormatType, RefType > & fmt
   )
   {
      return( formatlist_t< typename Container::const_iterator, FormatType, Outputter >
      (
         c.begin(),
         c.end(),
         fmt,
         out
      ));
   }
#endif
