// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMAT_OBJECTS__PAIR_OUTPUT__HPP
#define BOOST__IO__FORMAT_OBJECTS__PAIR_OUTPUT__HPP
#  include <boost/io/formatter.hpp>

#  include <utility>                    // std::pair
#  include <complex>                    // std::complex
#  include <boost/numeric/interval.hpp> // boost::numeric::interval
#  include <boost/rational.hpp>         // boost::rational
#  include <boost/compressed_pair.hpp>  // boost::compressed_pair

   namespace boost { namespace io
   {
      template
      <
         typename FormatType,
         typename Outputter1 = boost::io::basic_output,
         typename Outputter2 = boost::io::basic_output
      >
      class pair_output: public formatter_t< FormatType, pair_output
                                                         <
                                                            FormatType,
                                                            Outputter1,
                                                            Outputter2
                                                         > >
      {
         public:
            typedef pair_output< FormatType, Outputter1, Outputter2 >
                                                                     this_type;
            typedef typename formatter_t< FormatType, this_type >::traits_type
                                                                     traits_type;
         public:
            Outputter1                 firstout;
            Outputter2                 secondout;
         public: // standard library dual-valued types
            template< typename T1, typename T2, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const std::pair< T1, T2 > & p
                                  ) const
            {
               return( out( os, p.first, p.second ));
            }
            template< typename T, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const std::complex< T > & c
                                  ) const
            {
               return( out( os, c.real(), c.imag()));
            }
         public: // boost dual-valued types
            template< typename T, class Traits, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const boost::numeric::interval< T, Traits > & i
                                  ) const
            {
               return( out( os, i.lower(), i.upper()));
            }
            template< typename T, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const boost::rational< T > & r
                                  ) const
            {
               return( out( os, r.numerator(), r.denominator()));
            }
            template< typename T1, typename T2, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const boost::compressed_pair< T1, T2 > & cp
                                  ) const
            {
               return( out( os, cp.first(), cp.second()));
            }
         private: // internal implementation
            template< typename T1, typename T2, class OutputStream >
            inline OutputStream &                out
                                                 (
                                                    OutputStream & os,
                                                    const T1 &     first,
                                                    const T2 &     second
                                                 ) const
            {
               if( traits_type::isok( openlist()))
                  os << openlist();
               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               firstout( os, first );

               if( space_before_sep())
                  os << static_cast< typename OutputStream::char_type >( ' ' );
               if( traits_type::isok( separator()))
                  os << separator();
               if( space_after_sep())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               secondout( os, second );

               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );
               if( traits_type::isok( closelist()))
                  os << closelist();

               return( os );
            }
         public:
            inline           pair_output() throw();
            inline           pair_output( const Outputter1 & ) throw();
            inline           pair_output( const Outputter1 &, const Outputter2 & ) throw();
            inline           pair_output( const pair_output & o ) throw();
            template< typename RT2 >
            inline           pair_output
                             (
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               formatter_t< FormatType, pair_output >( fmt )
            {
            }
      };

      // pair_output generators

      inline pair_output< char >                 pairfmt();

      template< typename FormatType >
      inline pair_output< FormatType >           pairfmtex();
   }}

#  include "detail/pair_output.ipp"
#endif
