// (C) Copyright 2003: Reece H. Dunn

#ifndef BOOST__IO__FORMAT_OBJECTS__STATIC_NARY_OUTPUT__HPP
#define BOOST__IO__FORMAT_OBJECTS__STATIC_NARY_OUTPUT__HPP
#  include <boost/io/formatter.hpp>

#  include <boost/math/quaternion.hpp> // boost::math::quaternion
#  include <boost/math/octonion.hpp>   // boost::math::octonion
#  include <boost/tuple/tuple.hpp>     // boost::tuples::tuple

   namespace boost { namespace io
   {
      template
      <
         typename FormatType,
         typename Outputter = boost::io::basic_output
      >
      class static_nary_output: public formatter_t< FormatType, static_nary_output
                                                                <
                                                                    FormatType,
                                                                    Outputter
                                                                > >
      {
         public:
            typedef static_nary_output< FormatType, Outputter >      this_type;
            typedef typename formatter_t< FormatType, this_type >::traits_type
                                                                     traits_type;
         public:
            Outputter                  out;
         public: // nary-valued types
            template< typename T, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const boost::math::quaternion< T > & h
                                  ) const
            {
               outfirst( os, h.R_component_1());
               outnth(   os, h.R_component_2());
               outnth(   os, h.R_component_3());
               outnth(   os, h.R_component_4());
               return( outclose( os ));
            }
            template< typename T, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const boost::math::octonion< T > & o
                                  ) const
            {
               outfirst( os, o.R_component_1());
               outnth(   os, o.R_component_2());
               outnth(   os, o.R_component_3());
               outnth(   os, o.R_component_4());
               outnth(   os, o.R_component_5());
               outnth(   os, o.R_component_6());
               outnth(   os, o.R_component_7());
               outnth(   os, o.R_component_8());
               return( outclose( os ));
            }
         private: // internal implementation
            template< typename T, class OutputStream >
            inline OutputStream &                outfirst
                                                 (
                                                    OutputStream & os,
                                                    const T & t
                                                 ) const
            {
               if( traits_type::isok( openlist()))
                  os << openlist();

               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               return( out( os, t ));
            }
         private: // internal implementation
            template< typename T, class OutputStream >
            inline OutputStream &                outnth
                                                 (
                                                    OutputStream & os,
                                                    const T & t
                                                 ) const
            {
               if( space_before_sep())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               if( traits_type::isok( separator()))
                  os << separator();

               if( space_after_sep())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               return( out( os, t ));
            }
         private: // internal implementation
            template< class OutputStream >
            inline OutputStream &                outclose
                                                 (
                                                    OutputStream & os
                                                 ) const
            {
               if( space_around_list())
                  os << static_cast< typename OutputStream::char_type >( ' ' );

               if( traits_type::isok( closelist()))
                  os << closelist();

               return( os );
            }
         public:
            inline           static_nary_output() throw();
            inline           static_nary_output( const Outputter & ) throw();
            inline           static_nary_output( const static_nary_output & o ) throw();
            template< typename RT2 >
            inline           static_nary_output
                             (
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               formatter_t< FormatType, static_nary_output >( fmt )
            {
            }
      };

      // pair_output generators

      inline static_nary_output< char >          static_naryfmt();

      template< typename FormatType >
      inline static_nary_output< FormatType >    static_naryfmtex();
   }}

#  include "detail/static_nary_output.ipp"
#endif
