// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMATOB__HPP
#define BOOST__IO__FORMATOB__HPP
#  include <boost/io/format_objects.hpp>
#  include <boost/noncopyable.hpp>
#  include <iostream>

   namespace boost { namespace io
   {
      template< typename T, typename FormatType = char, typename Outputter = boost::io::basic_output >
      class formatob_t: public Outputter, public boost::noncopyable
      {
         public:
            const T &                  ob;
         public: // constructors
            inline           formatob_t( const T & ) throw();
            inline           formatob_t( const T &, const Outputter & ) throw();
            inline           formatob_t( const formatob_t & ) throw();
            template< typename RT2 >
            inline           formatob_t
                             (
                                const T & t,
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               Outputter( fmt ),
               ob(  t )
            {
            }
      };

      template< typename CharT, class TraitsT, typename T, typename FormatType, typename Outputter >
      inline std::basic_ostream< CharT, TraitsT > & operator<<
      (
         std::basic_ostream< CharT, TraitsT > &,
         const formatob_t< T, FormatType, Outputter > &
      );

      // format object generators

      template< typename T >
      inline formatob_t< T >                     formatob
                                                 (
                                                    const T &
                                                 );

      template< typename T, class Outputter >
      inline formatob_t< T, char, Outputter >    formatob
                                                 (
                                                    const T &,
                                                    const Outputter &
                                                 );
   }}

#  include "detail/formatob.ipp"
#endif
