// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__STL_IO__HPP
#define BOOST__STL_IO__HPP
#  include <boost/io/formatlist.hpp>
#  include <iostream>

#  include <vector>    // std::vector
#  include <list>      // std::list
#  include <deque>     // std::deque
#  include <map>       // std::map
#  include <set>       // std::set

   namespace std
   {
      // std::vector outputter

      template< typename CharT, class TraitsT, typename T, class Allocator >
      inline basic_ostream< CharT, TraitsT > & operator<<
      (
         basic_ostream< CharT, TraitsT > & os,
         const vector< T, Allocator >    & v
      )
      {
         return( os << boost::io::formatlist( v ));
      }

      // std::list outputter

      template< typename CharT, class TraitsT, typename T, class Allocator >
      inline basic_ostream< CharT, TraitsT > & operator<<
      (
         basic_ostream< CharT, TraitsT > & os,
         const list< T, Allocator >      & l
      )
      {
         return( os << boost::io::formatlist( l ));
      }

      // std::deque outputter

      template< typename CharT, class TraitsT, typename T, class Allocator >
      inline basic_ostream< CharT, TraitsT > & operator<<
      (
         basic_ostream< CharT, TraitsT > & os,
         const deque< T, Allocator >     & dq
      )
      {
         return( os << boost::io::formatlist( dq ));
      }

      // std::map outputter

      template
      <
         typename CharT, class TraitsT,
         typename KeyT, typename T, class Cmp, class Allocator
      >
      inline basic_ostream< CharT, TraitsT > & operator<<
      (
         basic_ostream< CharT, TraitsT >      & os,
         const map< KeyT, T, Cmp, Allocator > & m
      )
      {
         return
         (
            os << boost::io::formatlistfn
            (
               m,
               boost::io::pairfmt().format( '(', ')' )
            )
         );
      }

      // std::multimap outputter

      template
      <
         typename CharT, class TraitsT,
         typename KeyT, typename T, class Cmp, class Allocator
      >
      inline basic_ostream< CharT, TraitsT > & operator<<
      (
         basic_ostream< CharT, TraitsT >           & os,
         const multimap< KeyT, T, Cmp, Allocator > & mm
      )
      {
         return
         (
            os << boost::io::formatlistfn
            (
               mm,
               boost::io::pairfmt().format( '(', ')' )
            )
         );
      }

      // std::set outputter

      template
      <
         typename CharT, class TraitsT,
         typename KeyT, class Cmp, class Allocator
      >
      inline basic_ostream< CharT, TraitsT > & operator<<
      (
         basic_ostream< CharT, TraitsT >   & os,
         const set< KeyT, Cmp, Allocator > & s
      )
      {
         return( os << boost::io::formatlist( s ));
      }

      // std::multiset outputter

      template
      <
         typename CharT, class TraitsT,
         typename KeyT, class Cmp, class Allocator
      >
      inline basic_ostream< CharT, TraitsT > & operator<<
      (
         basic_ostream< CharT, TraitsT >        & os,
         const multiset< KeyT, Cmp, Allocator > & ms
      )
      {
         return( os << boost::io::formatlist( ms ));
      }
   }
#endif
