// (C) Copyright 2003: Reece H. Dunn

// Produce the same formatting as the examples created by John Torjo for his
// list outputter library/tutorial

#include <iostream>  // std::cout
#include <list>      // std::list

#include <boost/io/formatlist.hpp> // list formatting

template< class FormatType >
class position_output
{
   public:
      // needed for deduction of the nested format type
      typedef FormatType listfmt_type;
   private:
      long pos;
   public:
      template< typename T, class OutputStream >
      OutputStream & operator()( OutputStream & os, const T & value ) const
      {
         return( os << '[' << const_cast< long & >( pos )++ << "] " << value );
      }
   public:
      position_output(): pos( 0 ){}
};

int main()
{
   // names - the data to be formatted

   std::list< std::string >            names;
   names.push_back( "John" );
   names.push_back( "James" );
   names.push_back( "Corina" );

   // fmt_char; fmt_str - helper format configurations for the examples
   // ==> These encode frequently used configurations

   boost::io::formatter< char >        fmt_char;
   fmt_char.format( '\0', '\0' ).space( false, false, false );

   boost::io::formatter< char * >      fmt_str;
   fmt_str.format( "\0", "\0" ).space( false, false, false );

   // [1]: John, James, Corina

   std::cout << "[1]: "
             << boost::io::formatlist( names )
                .format( '\0', '\0' ).space( false )
             << '\n';

   // [2]: {John}, {James}, {Corina}

   std::cout << "[2]: "
             << boost::io::formatlistex< char * >( names )
                .format( "{", "}", "}, {" )
                .space( false, false, false )
             << '\n';

   std::cout << "[2]: "
             << boost::io::formatlist( names, fmt_str )
                .format( "{", "}", "}, {" )
             << '\n';

   // [3]: {[0] John}, {[1] James}, {[2] Corina}

   std::cout << "[3]: "
             << boost::io::formatlistfn( names, position_output< char * >())
                .format( "{", "}", "}, {" )
                .space( false, false, false )
             << '\n';

   std::cout << "[3]: "
             << boost::io::formatlistfn2( names, position_output< char * >(), fmt_str )
                .format( "{", "}", "}, {" )
             << '\n';

   // [4]: [0] John, [1] James, [2] Corina

   std::cout << "[4]: "
             << boost::io::formatlistfn( names, position_output< char >())
                .format( '\0', '\0' ).space( false )
             << '\n' << '\n';

   /*
      [5]:
      [John]
      [James]
      [Corina]
   */

   std::cout << "[5]:\n"
             << boost::io::formatlistex< char * >( names )
                .format( "[", "]", "]\n[" )
                .space( false, false, false )
             << '\n' << '\n';

   std::cout << "[5]:\n"
             << boost::io::formatlist( names, fmt_str )
                .format( "[", "]", "]\n[" )
             << '\n' << '\n';

   /*
      [6]:
      [0] John
      [1] James
      [2] Corina
   */

   std::cout << "[6]:\n"
             << boost::io::formatlistfn( names, position_output< char >())
                .format( '\0', '\0', '\n' )
                .space( false, false, false )
             << '\n' << '\n';

   std::cout << "[6]:\n"
             << boost::io::formatlistfn2( names, position_output< char >(), fmt_char )
                .format( '\n' )
             << '\n' << '\n';

   /*
      [7]:
      '[0] John'
      '[1] James'
      '[2] Corina'
   */

   std::cout << "[7]:\n"
             << boost::io::formatlistfn( names, position_output< char * >())
                .format( "'", "'", "'\n'" )
                .space( false, false, false )
             << '\n' << '\n';

   std::cout << "[7]:\n"
             << boost::io::formatlistfn2( names, position_output< char * >(), fmt_str )
                .format( "'", "'", "'\n'" )
             << '\n' << '\n';

   return( 0 );
}
